@@ -95,56 +95,29 @@ class requestManager:
         - It provides methods for authentication, response handling, and error management.
     """
 class requestManager:
-    def __init__(self,
-                 url=None,
-                 source_code=None,
-                 url_mgr=None,
-                 network_manager=None,
-                 ua_mgr=None,
-                 ssl_manager=None,
-                 ssl_options=None,
-                 tls_adapter=None,
-                 user_agent=None,
-                 proxies=None,
-                 headers=None,
-                 cookies=None,
-                 session=None,
-                 adapter=None,
-                 protocol=None,
-                 ciphers=None,
-                 spec_login=False,
-                 login_referer=None,
-                 login_user_agent=None,
-                 auth=None,
-                 login_url=None,
-                 email=None,
-                 password=None,
-                 checkbox=None,
-                 dropdown=None,
-                 certification=None,
-                 stream=False,
-                 timeout=None,
-                 last_request_time=None,
-                 max_retries=None,
-                 request_wait_limit=None):
+    def __init__(self, url=None, source_code=None, url_mgr=None, network_manager=None,
+                 ua_mgr=None, ssl_manager=None, ssl_options=None, tls_adapter=None,
+                 user_agent=None, proxies=None, headers=None, cookies=None, session=None,
+                 adapter=None, protocol=None, ciphers=None, spec_login=False,
+                 login_referer=None, login_user_agent=None, auth=None, login_url=None,
+                 email=None, password=None, checkbox=None, dropdown=None,
+                 certification=None, stream=False, timeout=None, last_request_time=None,
+                 max_retries=None, request_wait_limit=None):
+
         self.url_mgr = get_url_mgr(url=url, url_mgr=url_mgr)
         self.url = get_url(url=url, url_mgr=self.url_mgr)
-        self._url_mgr = self.url_mgr
-        self._url = self.url
 
-        # User-Agent manager
+        # UA/headers
         self.ua_mgr = ua_mgr or get_ua_mgr(user_agent=user_agent)
         self.user_agent = self.ua_mgr.user_agent
-        self.headers = headers or self.ua_mgr.generate_headers(url=self.url)  # fixed
+        # generate realistic headers tied to this URL
+        self.headers = headers or self.ua_mgr.generate_for_url(self.url)
 
         # TLS / SSL / Network
         self.ciphers = ciphers or CipherManager().ciphers_string
         self.certification = certification
         self.ssl_manager = ssl_manager or SSLManager(ciphers=self.ciphers)
-        self.tls_adapter = tls_adapter or TLSAdapter(
-            ssl_manager=self.ssl_manager,
-            certification=self.certification
-        )
+        self.tls_adapter = tls_adapter or TLSAdapter(ssl_manager=self.ssl_manager, certification=self.certification)
         self.network_manager = network_manager or NetworkManager(
             user_agent_manager=self.ua_mgr,
             ssl_manager=self.ssl_manager,
@@ -157,10 +130,15 @@ class requestManager:
             ssl_options=ssl_options
         )
 
-        # Session and defaults
+        # Session
         self.session = session or requests.Session()
-        self.proxies = self.network_manager.proxies
-        self.cookies = self.network_manager.cookies
+        self.session.proxies = self.network_manager.proxies
+        self.session.headers.update(self.headers)
+        self.session.mount("https://", self.network_manager.tls_adapter)
+        self.session.mount("http://", HTTPAdapter())
+        if auth:
+            self.session.auth = auth
+
         self.protocol = protocol or 'https://'
         self.timeout = timeout
         self.auth = auth
@@ -174,13 +152,11 @@ class requestManager:
         self.login_referer = login_referer
         self.stream = bool(stream)
 
-        self.initialize_session()
-
         self.last_request_time = last_request_time
         self.max_retries = max_retries or 3
         self.request_wait_limit = request_wait_limit or 1.5
 
-        # Response placeholders
+        # Response placeholders ...
         self._response = None
         self.status_code = None
         self.source_code = None
@@ -206,6 +182,7 @@ class requestManager:
         self.url_mgr.update_url(url=url)
         self.re_initialize()
 
+    # --- re_initialize, update_url_mgr, update_url unchanged except: ---
     def re_initialize(self):
         self._response = None
         if self.url_mgr.url is not None:
@@ -219,13 +196,14 @@ class requestManager:
         self._response_data = None
         self.process_response_data()
 
+
     @property
     def response(self):
-        """Lazy-loading of response."""
         if self._response is None and self.url_mgr.url is not None:
             self._response = self.fetch_response()
         return self._response
 
+
     def authenticate(self, session, login_url=None, email=None, password=None, checkbox=None, dropdown=None):
         login_urls = login_url or [self.url_mgr.url, self.url_mgr.domain, self.url_mgr.url_join(url=self.url_mgr.domain, path='login'), self.url_mgr.url_join(url=self.url_mgr.domain, path='auth')]
         s = session
@@ -273,18 +251,9 @@ class requestManager:
         return s
 
     def initialize_session(self):
-        s = self.session
-        if self.auth:
-            s = self.auth
-        elif self.spec_login:
-            s = self.spec_auth(session=s, email=self.email, password=self.password, login_url=self.login_url, login_referer=self.login_referer, login_user_agent=self.login_user_agent)
-        elif any([self.password, self.email, self.login_url, self.checkbox, self.dropdown]):
-            s = self.authenticate(session=s, login_url=self.login_url, email=self.email, password=self.password, checkbox=self.checkbox, dropdown=self.dropdown)
-        s.proxies = self.proxies
- 
-        s.headers.update(self.headers)
-        s.mount(self.protocol, self.network_manager.tls_adapter)
-        return s
+        # Already done in __init__; keep for API compatibility
+        return self.session
+
 
     def process_response_data(self):
         """Processes the fetched response data."""
@@ -358,36 +327,33 @@ class requestManager:
             self.react_source_code.append(self.source_code)
         return self.react_source_code
 
+    def initialize_session(self):
+        # Already done in __init__; keep for API compatibility
+        return self.session
+
+    def fetch_response(self):
+        return self.try_request()
+
     def get_status(self, url: str = None) -> int | None:
-        """
-        Gets the HTTP status code of the given URL.
-        """
         url = url or self.url_mgr.url
         if url is None:
             return None
         try:
-            response = requests.head(url, timeout=5)
-            return response.status_code
+            r = self.session.head(url, timeout=5)
+            return r.status_code
         except requests.RequestException:
             return None
 
     def wait_between_requests(self):
-        """
-        Wait between requests based on the request_wait_limit.
-        """
         if self.last_request_time:
             sleep_time = self.request_wait_limit - (time.time() - self.last_request_time)
             if sleep_time > 0:
-                logging.info(f"Sleeping for {sleep_time:.2f} seconds.")
+                logger.info("Sleeping for %.2f seconds.", sleep_time)
                 time.sleep(sleep_time)
 
     def make_request(self):
-        """
-        Make a request and handle potential errors, with retries.
-        """
         if self.url_mgr.url is None:
             return None
-
         self.wait_between_requests()
         for _ in range(self.max_retries):
             try:
@@ -395,24 +361,22 @@ class requestManager:
                 if self._response:
                     if not isinstance(self._response, (str, bytes)):
                         self.status_code = self._response.status_code
-                        if self._response.status_code == 200:
+                        if self.status_code == 200:
                             self.last_request_time = time.time()
                             return self._response
-                        elif self._response.status_code == 429:
-                            logging.warning(f"Rate limited by {self.url_mgr.url}. Retrying...")
+                        if self.status_code == 429:
+                            logger.warning("429 from %s. Retrying...", self.url_mgr.url)
                             time.sleep(5)
                     else:
-                        # String/bytes from Selenium path
                         self.status_code = 200
                         return self._response
             except requests.Timeout as e:
-                logging.error(f"Request to {self.url_mgr.url} timed out: {e}")
+                logger.error("Timeout %s: %s", self.url_mgr.url, e)
             except requests.ConnectionError:
-                logging.error(f"Connection error for URL {self.url_mgr.url}.")
+                logger.error("Connection error %s", self.url_mgr.url)
             except requests.RequestException as e:
-                logging.error(f"Request exception for URL {self.url_mgr.url}: {e}")
-
-        logging.error(f"Failed to retrieve content from {self.url_mgr.url} after {self.max_retries} retries")
+                logger.error("Request exception %s: %s", self.url_mgr.url, e)
+        logger.error("Failed to retrieve content from %s after %d retries", self.url_mgr.url, self.max_retries)
         return None
 
     def try_request(self) -> requests.Response | str | bytes | None:
@@ -439,14 +403,16 @@ class requestManager:
         except requests.RequestException as e:
             logging.error(f"Requests fallback also failed for {self.url_mgr.url}: {e}")
             return None
+    @property
+    def url(self):
+        return self.url_mgr.url
 
-        @property
-        def url(self):
-            return self.url_mgr.url
-
-        @url.setter
-        def url(self, new_url):
-            self._url = new_url
+    @url.setter
+    def url(self, new_url):
+        if self.url_mgr:
+            self.url_mgr.update_url(new_url)
+        else:
+            self.url_mgr = urlManager(new_url)
 class SafeRequestSingleton:
     _instance = None
     @staticmethod
