@@ -1,6 +1,6 @@
 from .imports import *
 from .path_utils import *
-from abstract_utilities import make_list
+from abstract_utilities import make_list,get_any_value
 _LOCK = threading.RLock()
 # ---------------- Registry ---------------- #
 def get_video_url(url=None, video_url=None):
@@ -215,21 +215,21 @@ class VideoDownloader:
                  registry=None, force_refresh=False,
                  flat_layout: bool = False,video_url =None):
 
-        self.video_url = get_corrected_url(url or video_url) 
-        self.video_urls = make_list( self.video_url)
-        self.download_directory = get_video_root(download_directory)
-        self.registry = registry or infoRegistry(video_root=self.download_directory,flat_layout=flat_layout)
+        self.video_url = get_video_url(url or video_url )
+        self.video_urls = make_list(self.video_url)
+        self.video_id =  generate_video_id(self.video_url)
+        self.registry = registry or infoRegistry(video_root=download_directory,flat_layout=flat_layout)
         self.ydl_opts = ydl_opts or {}
         self.get_download = download_video
         self.user_agent = user_agent
         self.video_extention = video_extention
-        
+        self.download_directory = get_video_root(download_directory)
         self.output_filename = output_filename
         self.force_refresh = force_refresh
         self.flat_layout = flat_layout   # 🔑
         self.video_path=video_path
-        self.info = self.registry.get_video_info(url=self.video_url, force_refresh=self.force_refresh)
-        
+        self.info = self.registry.get_video_info(url=self.video_url,video_id=self.video_id,video_path=self.video_path, force_refresh=self.force_refresh) or {}
+        self.video_path=self.info.get('video_path') or self.video_path
         
         self.monitoring = True
         self.pause_event = threading.Event()
@@ -299,7 +299,7 @@ class VideoDownloader:
                          video_path=None, output_filename=None, ydl_opts=None,
                          registry=None, force_refresh=False,
                          flat_layout: bool = False,video_url =None):
-        video_url = get_video_url(url or video_url or self.url or self.video_url)
+        video_url = get_video_url(url or video_url  or self.video_url)
         self.info = self.registry.get_video_info(url=video_url, force_refresh=self.force_refresh)
         info_video_path = self.info.get('video_path')
         if info_video_path and os.path.isfile(info_video_path) and not force_refresh:
